/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.launcher.util;

import java.io.File;
import java.io.IOException;
import java.lang.management.ManagementFactory;
import java.lang.management.RuntimeMXBean;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.io.FileUtils;
import org.ow2.petals.kernel.api.server.util.SystemUtil;

/**
 * A container locker; When the container is started, no other container can be
 * started on the same context.
 * <p>
 * Note that the locker cannot be based on the PID of the running program as
 * JAVA doesn't provide any method to get it.
 * 
 * @author chamerling - eBM WebSourcing
 * 
 */
public class Locker {

    private final File file;

    private static final String FILE_NAME = "locked";

    private static final Pattern pattern = Pattern.compile("^([0-9]+)@.+$",
            Pattern.CASE_INSENSITIVE);

    /**
     * Creates a new {@link Locker}
     * 
     * @param rootPath
     */
    public Locker(File rootPath) {
        this.file = new File(rootPath, FILE_NAME);
    }

    /**
     * Test if the lock file is present and if its refered PID is running
     * 
     * @return the running PID as String of {@code null}
     */
    public String isLocked() {
        String pid = null;
        if (this.file.exists()) {
            try {
                final String tempPid = FileUtils.readFileToString(this.file);
                final String osName = System.getProperty("os.name");
                if (osName.toLowerCase().contains("linux") || osName.toLowerCase().contains("mac")) {
                    final Process process = Runtime.getRuntime().exec("kill -0 " + tempPid);
                    if (process.waitFor() == 0) {
                        pid = tempPid;
                    }
                } else if (osName.toLowerCase().contains("windows")) {
                    final Process process = Runtime.getRuntime()
                            .exec(
                                    SystemUtil.getPetalsInstallDirectory() + "\\bin\\pslist.exe /accepteula "
                                            + tempPid);
                    if (process.waitFor() == 0) {
                        pid = tempPid;
                    }
                }
            } catch (final Exception e) {
                // consider as unlocked
            }
        }

        return pid;
    }

    /**
     * Create the lock file
     * 
     */
    public void lock() {
        try {
            if (this.file.exists()) {
                this.file.delete();
            }
            this.file.createNewFile();
            // get PID trick from
            // http://golesny.de/wiki/code:javahowtogetpid
            final RuntimeMXBean runtimeMBean = ManagementFactory.getRuntimeMXBean();
            final String processName = runtimeMBean.getName();
            final String pid = tryGetPID(processName);
            if (pid != null) {
                // create the file only if the PID has been retrieved
                FileUtils.writeByteArrayToFile(this.file, pid.getBytes());
            }
        } catch (final IOException e) {
            System.out.println("Failed to create lock file : " + e.getMessage());
        }

    }

    /**
     * Remove the lock file
     */
    public void unlock() {
        this.file.delete();
    }

    private static final String tryGetPID(String processName) {
        final Matcher matcher = pattern.matcher(processName);
        if (matcher.matches()) {
            return matcher.group(1);
        }
        return null;

    }
}
